@@ -1,5 +1,5 @@
 /*
-    Copyright (c) 2013 Christian Schoenebeck
+    Copyright (c) 2013 - 2017 Christian Schoenebeck
     
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
@@ -24,6 +24,18 @@ extern "C" {
  */
 void macHelperRunCFuncOnMainThread(void (*fn)(void* info), void* info);
 
+/**
+ * Calling this function ensures that the app windows really becomes visible
+ * in front of all other windows if the window is raised.
+ */
+void macRaiseAppWindow();
+
+/**
+ * Returns @c true if this Mac runs at least OS X 10.6 ("Snow Leopard") or
+ * higher.
+ */
+bool macIsMinMac10_6();
+
 #if defined(__cplusplus)
 }
 #endif // __cplusplus
