@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2014 Andreas Persson
+ * Copyright (C) 2006-2017 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,14 +17,29 @@
  * 02110-1301 USA.
  */
 
+#include "compat.h"
+// threads.h must be included first to be able to build with
+// G_DISABLE_DEPRECATED
+#if (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION == 31 && GLIBMM_MICRO_VERSION >= 2) || \
+    (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION > 31) || GLIBMM_MAJOR_VERSION > 2
+#include <glibmm/threads.h>
+#endif
+
 #include "dimensionmanager.h"
 
-#include <gtkmm/stock.h>
+#if HAS_GTKMM_STOCK
+# include <gtkmm/stock.h>
+#endif
 #include <gtkmm/messagedialog.h>
 #include <gtkmm/dialog.h>
 #include <gtkmm/comboboxtext.h>
 #include <gtkmm/spinbutton.h>
-#include <gtkmm/table.h>
+#include <gtkmm/label.h>
+#if USE_GTKMM_GRID
+# include <gtkmm/grid.h>
+#else
+# include <gtkmm/table.h>
+#endif
 
 #include "global.h"
 #include "compat.h"
@@ -236,11 +251,26 @@ void IntSetCellRenderer::valueChanged() {
 }
 
 DimensionManager::DimensionManager() :
-addButton(Gtk::Stock::ADD), removeButton(Gtk::Stock::REMOVE),
-allRegionsCheckBox(_("All Regions"))
+#if HAS_GTKMM_STOCK
+    addButton(Gtk::Stock::ADD), removeButton(Gtk::Stock::REMOVE),
+#else
+    addButton(_("_Add"), true), removeButton(_("_Remove"), true),
+#endif
+    allRegionsCheckBox(_("All Regions"))
 {
     ignoreColumnClicked = true;
 
+    if (!Settings::singleton()->autoRestoreWindowDimension) {
+        set_default_size(630, 250);
+        set_position(Gtk::WIN_POS_MOUSE);
+    }
+
+#if !HAS_GTKMM_STOCK
+    // see : https://specifications.freedesktop.org/icon-naming-spec/icon-naming-spec-latest.html
+    addButton.set_icon_name("list-add");
+    removeButton.set_icon_name("list-remove");
+#endif
+
     set_title(_("Dimensions of selected Region"));
     add(vbox);
     scrolledWindow.add(treeView);
@@ -248,7 +278,11 @@ allRegionsCheckBox(_("All Regions"))
     scrolledWindow.show();
     vbox.pack_start(buttonBox, Gtk::PACK_SHRINK);
     buttonBox.set_layout(Gtk::BUTTONBOX_END);
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+    buttonBox.set_margin(5);
+#else
     buttonBox.set_border_width(5);
+#endif
     buttonBox.show();
     buttonBox.pack_start(allRegionsCheckBox, Gtk::PACK_EXPAND_PADDING);
     buttonBox.pack_start(addButton, Gtk::PACK_SHRINK);
@@ -288,9 +322,9 @@ allRegionsCheckBox(_("All Regions"))
         sigc::mem_fun(*this, &DimensionManager::onAllRegionsCheckBoxToggled)
     );
 
+#if HAS_GTKMM_SHOW_ALL_CHILDREN
     show_all_children();
-    
-    resize(460,300);
+#endif
 }
 
 bool DimensionManager::allRegions() const {
@@ -384,11 +418,26 @@ void DimensionManager::set_region(gig::Region* region) {
 }
 
 void DimensionManager::onColumnClicked() {
+    printf("DimensionManager::onColumnClicked()\n");
+
+    //FIXME: BUG: this method is currently very unreliably called, it should actually be called when the user selects another column, it is ATM however also called when the table content changed programmatically causing the dialog below to popup at undesired times !
+
     //HACK: Prevents that onColumnClicked() gets called multiple times or at times where it is not desired
     if (ignoreColumnClicked) {
         ignoreColumnClicked = false;
         return;
     }
+#if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION >= 18) || GTKMM_MAJOR_VERSION > 2
+    // prevents app to crash if this dialog is closed
+    if (!get_visible())
+        return;
+#else
+# warning Your GTKMM version is too old; dimension manager dialog might crash when changing a dimension type !
+#endif
+
+#if (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION >= 8) || GTKMM_MAJOR_VERSION > 3
+    if (!is_visible()) return;
+#endif
 
     Gtk::TreeModel::Path path;
     Gtk::TreeViewColumn* focus_column;
@@ -396,6 +445,7 @@ void DimensionManager::onColumnClicked() {
     //const int row = path[0];
     if (focus_column == treeView.get_column(0)) {
         Gtk::TreeModel::iterator it = treeView.get_model()->get_iter(path);
+        if (!it) return;
         Gtk::TreeModel::Row row = *it;
         gig::dimension_t oldType = row[tableModel.m_type];
 
@@ -413,7 +463,11 @@ void DimensionManager::onColumnClicked() {
                 count++;
             }
         }
+#if USE_GTKMM_GRID
+        Gtk::Grid table;
+#else
         Gtk::Table table(1, 2);
+#endif
         Gtk::Label labelDimType(_("Dimension:"), Gtk::ALIGN_START);
         Gtk::ComboBox comboDimType;
         comboDimType.set_model(refComboModel);
@@ -421,15 +475,27 @@ void DimensionManager::onColumnClicked() {
         comboDimType.pack_start(comboModel.m_type_name);
         table.attach(labelDimType, 0, 1, 0, 1);
         table.attach(comboDimType, 1, 2, 0, 1);
+#if USE_GTKMM_BOX
+        dialog.get_content_area()->pack_start(table);
+#else
         dialog.get_vbox()->pack_start(table);
+#endif
 
+#if HAS_GTKMM_STOCK
         dialog.add_button(Gtk::Stock::OK, 0);
         dialog.add_button(Gtk::Stock::CANCEL, 1);
+#else
+        dialog.add_button(_("_OK"), 0);
+        dialog.add_button(_("_Cancel"), 1);
+#endif
+#if HAS_GTKMM_SHOW_ALL_CHILDREN
         dialog.show_all_children();
+#endif
         
         comboDimType.set_active(oldTypeIndex);
 
         if (!dialog.run()) { // OK selected ...
+            ignoreColumnClicked = true;
             Gtk::TreeModel::iterator iterType = comboDimType.get_active();
             if (!iterType) return;
             Gtk::TreeModel::Row rowType = *iterType;
@@ -438,7 +504,6 @@ void DimensionManager::onColumnClicked() {
             gig::dimension_t newType = static_cast<gig::dimension_t>(iTypeID);
             if (newType == oldType) return;
             //printf("change 0x%x -> 0x%x\n", oldType, newType);
-            ignoreColumnClicked = true;
 
             // assemble the list of regions where the selected dimension type
             // shall be changed
@@ -497,6 +562,10 @@ void DimensionManager::onColumnClicked() {
                 msg.run();
             }
         }
+    } else if (focus_column == treeView.get_column(1) || focus_column == treeView.get_column(2)) {
+        Glib::ustring txt = _("Right-click on a specific dimension zone of the dimension region selector to delete or split that particular dimension zone!");
+        Gtk::MessageDialog msg(*this, txt, false, Gtk::MESSAGE_INFO);
+        msg.run();
     }
 }
 
@@ -513,17 +582,32 @@ void DimensionManager::addDimension() {
             row[comboModel.m_type_name] = sType;
         }
     }
+#if USE_GTKMM_GRID
+    Gtk::Grid table;
+#else
     Gtk::Table table(2, 2);
+#endif
     Gtk::Label labelDimType(_("Dimension:"), Gtk::ALIGN_START);
     Gtk::ComboBox comboDimType;
     comboDimType.set_model(refComboModel);
     comboDimType.pack_start(comboModel.m_type_id);
     comboDimType.pack_start(comboModel.m_type_name);
     Gtk::Label labelZones(_("Zones:"), Gtk::ALIGN_START);
+#if USE_GTKMM_GRID
+    table.attach(labelDimType, 0, 0);
+    table.attach(comboDimType, 1, 0);
+    table.attach(labelZones, 0, 1);
+#else
     table.attach(labelDimType, 0, 1, 0, 1);
     table.attach(comboDimType, 1, 2, 0, 1);
     table.attach(labelZones, 0, 1, 1, 2);
+#endif
+
+#if USE_GTKMM_BOX
+    dialog.get_content_area()->pack_start(table);
+#else
     dialog.get_vbox()->pack_start(table);
+#endif
 
     // number of zones: use a combo box with fix values for gig
     // v2 and a spin button for v3
@@ -553,9 +637,16 @@ void DimensionManager::addDimension() {
         table.attach(spinZones, 1, 2, 1, 2);
     }
 
+#if HAS_GTKMM_STOCK
     dialog.add_button(Gtk::Stock::OK, 0);
     dialog.add_button(Gtk::Stock::CANCEL, 1);
+#else
+    dialog.add_button(_("_OK"), 0);
+    dialog.add_button(_("_Cancel"), 1);
+#endif
+#if HAS_GTKMM_SHOW_ALL_CHILDREN
     dialog.show_all_children();
+#endif
 
     if (!dialog.run()) { // OK selected ...
         Gtk::TreeModel::iterator iterType = comboDimType.get_active();
