@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2015 Andreas Persson
+ * Copyright (C) 2007-2017 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,12 +17,28 @@
  * 02110-1301 USA.
  */
 
+#include "compat.h"
+// threads.h must be included first to be able to build with
+// G_DISABLE_DEPRECATED
+#if (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION == 31 && GLIBMM_MICRO_VERSION >= 2) || \
+    (GLIBMM_MAJOR_VERSION == 2 && GLIBMM_MINOR_VERSION > 31) || GLIBMM_MAJOR_VERSION > 2
+#include <glibmm/threads.h>
+#endif
+
 #include "gigedit.h"
 
+#if GTKMM_MAJOR_VERSION < 3
+#include <gdkmm/region.h>
+#endif
 #include <glibmm/dispatcher.h>
 #include <glibmm/main.h>
+#include <glibmm/miscutils.h>
 #include <gtkmm/main.h>
 
+#ifdef WIN32
+#include <gtkmm/icontheme.h>
+#endif
+
 #if defined(__APPLE__)
 # include <CoreFoundation/CoreFoundation.h>
 # include "MacHelper.h"
@@ -35,11 +51,10 @@
 #ifdef __APPLE__
 #include <dlfcn.h>
 #include <glibmm/fileutils.h>
-#include <glibmm/miscutils.h>
 #endif
 
 //TODO: (hopefully) just a temporary nasty hack for launching gigedit on the main thread on Mac (see comments below in this file for details)
-#if defined(__APPLE__) // the following global external variables are defined in LinuxSampler's global_private.cpp ...
+#if defined(__APPLE__) && HAVE_LINUXSAMPLER // the following global external variables are defined in LinuxSampler's global_private.cpp ...
 extern bool g_mainThreadCallbackSupported;
 extern void (*g_mainThreadCallback)(void* info);
 extern void* g_mainThreadCallbackInfo;
@@ -58,7 +73,8 @@ namespace {
 //
 class GigEditState : public sigc::trackable {
 public:
-    GigEditState(GigEdit* parent) : parent(parent), instrument(NULL) { }
+    GigEditState(GigEdit* parent) :
+        window(0), parent(parent), instrument(0) { }
     void run(gig::Instrument* pInstrument);
 
     MainWindow* window;
@@ -110,6 +126,8 @@ private:
 
 #ifdef WIN32
 HINSTANCE gigedit_dll_handle = 0;
+std::string gigedit_datadir;
+bool gigedit_installdir_is_parent = false;
 #endif
 
 #ifdef __APPLE__
@@ -128,7 +146,11 @@ void init_app() {
         // under the same dir as the gigedit dylib is installed in.
         Dl_info info;
         if (dladdr((void*)&init_app, &info)) {
+#ifdef CONFIG_FORCE_GTK_LIBDIR
+            std::string libdir = CONFIG_FORCE_GTK_LIBDIR;
+#else
             std::string libdir = Glib::path_get_dirname(info.dli_fname);
+#endif
 
             if (Glib::getenv("PANGO_SYSCONFDIR") == "" &&
                 Glib::file_test(Glib::build_filename(libdir,
@@ -163,21 +185,41 @@ void init_app() {
                                  ".local/share").c_str(), 0777);
 #endif // __APPLE__
 
-//FIXME: for some reason AC GETTEXT check fails on the Mac cross compiler?
-#if (HAVE_GETTEXT || defined(__APPLE__))
+#ifdef WIN32
+        // Find the data directory: the linuxsampler installer puts
+        // the binaries in sub directories "32" and "64", so the share
+        // directory is located in the parent of the directory of the
+        // binaries.
 
-  #ifdef WIN32
-    #if GLIB_CHECK_VERSION(2, 16, 0)
+  #if GLIB_CHECK_VERSION(2, 16, 0)
         gchar* root =
             g_win32_get_package_installation_directory_of_module(gigedit_dll_handle);
-    #else
+  #else
         gchar* root =
             g_win32_get_package_installation_directory(NULL, NULL);
-    #endif
-        gchar* temp = g_build_filename(root, "/share/locale", NULL);
+  #endif
+        std::string installdir(root);
         g_free(root);
-        gchar* localedir = g_win32_locale_filename_from_utf8(temp);
-        g_free(temp);
+        std::string basename = Glib::path_get_basename(installdir);
+        if (basename == "32" || basename == "64") {
+            installdir = Glib::path_get_dirname(installdir);
+            gigedit_installdir_is_parent = true;
+        }
+        gigedit_datadir = Glib::build_filename(installdir, "share");
+
+        // the file dialogs need glib-2.0/schemas/gschemas.compiled
+        if (gigedit_installdir_is_parent) {
+            Glib::setenv("GSETTINGS_SCHEMA_DIR",
+                         Glib::build_filename(gigedit_datadir,
+                                              "glib-2.0/schemas"));
+        }
+#endif
+
+//FIXME: for some reason AC GETTEXT check fails on the Mac cross compiler?
+#if (HAVE_GETTEXT || defined(__APPLE__))
+  #ifdef WIN32
+        std::string temp = Glib::build_filename(gigedit_datadir, "locale");
+        gchar* localedir = g_win32_locale_filename_from_utf8(temp.c_str());
         bindtextdomain(GETTEXT_PACKAGE, localedir);
         g_free(localedir);
   #elif !defined(__APPLE__)
@@ -195,6 +237,25 @@ void init_app() {
     }
 }
 
+void init_app_after_gtk_init() {
+//FIXME: for some reason AC GETTEXT check fails on the Mac cross compiler?
+#if (/*HAVE_GETTEXT &&*/ defined(__APPLE__))
+    // Gtk::Main binds the gtk locale to a possible non-existent
+    // directory. If we have bundled gtk locale files, we rebind here,
+    // after the Gtk::Main constructor.
+    if (!gigedit_localedir.empty()) {
+        bindtextdomain("gtk20", gigedit_localedir.c_str());
+    }
+#endif
+
+#ifdef WIN32
+    if (gigedit_installdir_is_parent) {
+        std::string icon_dir = Glib::build_filename(gigedit_datadir, "icons");
+        Gtk::IconTheme::get_default()->append_search_path(icon_dir);
+    }
+#endif
+}
+
 void connect_signals(GigEdit* gigedit, MainWindow* mainwindow) {
     // the signals of the "GigEdit" class are actually just proxies, that
     // is they simply forward the signals of the internal classes to the
@@ -238,6 +299,12 @@ void connect_signals(GigEdit* gigedit, MainWindow* mainwindow) {
     mainwindow->signal_switch_sampler_instrument().connect(
         gigedit->signal_switch_sampler_instrument().make_slot()
     );
+    mainwindow->signal_script_to_be_changed.connect(
+        gigedit->signal_script_to_be_changed.make_slot()
+    );
+    mainwindow->signal_script_changed.connect(
+        gigedit->signal_script_changed.make_slot()
+    );
 }
 
 } // namespace
@@ -249,22 +316,23 @@ GigEdit::GigEdit() {
 int GigEdit::run(int argc, char* argv[]) {
     init_app();
 
+#if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION < 89 || (GTKMM_MINOR_VERSION == 89 && GTKMM_MICRO_VERSION < 4))) // GTKMM < 3.89.4
     Gtk::Main kit(argc, argv);
-
-//FIXME: for some reason AC GETTEXT check fails on the Mac cross compiler?
-#if (/*HAVE_GETTEXT &&*/ defined(__APPLE__))
-    // Gtk::Main binds the gtk locale to a possible non-existent
-    // directory. If we have bundled gtk locale files, we rebind here,
-    // after the Gtk::Main constructor.
-    if (!gigedit_localedir.empty()) {
-        bindtextdomain("gtk20", gigedit_localedir.c_str());
-    }
+#else
+    Glib::RefPtr<Gtk::Application> app =
+        Gtk::Application::create("org.linuxsampler.gigedit");
 #endif
+    init_app_after_gtk_init();
 
     MainWindow window;
     connect_signals(this, &window);
     if (argc >= 2) window.load_file(argv[1]);
+#if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION < 89 || (GTKMM_MINOR_VERSION == 89 && GTKMM_MICRO_VERSION < 4))) // GTKMM < 3.89.4
     kit.run(window);
+#else
+    app->run(window, argc, argv);
+#endif
+    
     return 0;
 }
 
@@ -280,13 +348,13 @@ int GigEdit::run(gig::Instrument* pInstrument) {
 
 void GigEdit::on_note_on_event(int key, int velocity) {
     if (!this->state) return;
-    GigEditState* state = (GigEditState*) this->state;
+    GigEditState* state = static_cast<GigEditState*>(this->state);
     state->window->signal_note_on().emit(key, velocity);
 }
 
 void GigEdit::on_note_off_event(int key, int velocity) {
     if (!this->state) return;
-    GigEditState* state = (GigEditState*) this->state;
+    GigEditState* state = static_cast<GigEditState*>(this->state);
     state->window->signal_note_off().emit(key, velocity);
 }
 
@@ -372,23 +440,28 @@ void GigEditState::close_window() {
     close.signal();
 }
 
+#if defined(WIN32) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 2))
+// make sure stack is 16-byte aligned for SSE instructions
+__attribute__((force_align_arg_pointer))
+#endif
 void GigEditState::main_loop_run(Cond* initialized) {
     int argc = 1;
     const char* argv_c[] = { "gigedit" };
     char** argv = const_cast<char**>(argv_c);
+#if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION < 89 || (GTKMM_MINOR_VERSION == 89 && GTKMM_MICRO_VERSION < 4))) // GTKMM < 3.89.4
     Gtk::Main main_loop(argc, argv);
-//FIXME: for some reason AC GETTEXT check fails on the Mac cross compiler?
-#if (/*HAVE_GETTEXT &&*/ defined(__APPLE__))
-    if (!gigedit_localedir.empty()) {
-        bindtextdomain("gtk20", gigedit_localedir.c_str());
-    }
 #endif
+    init_app_after_gtk_init();
 
     dispatcher = new Glib::Dispatcher();
     dispatcher->connect(sigc::ptr_fun(&GigEditState::open_window_static));
     initialized->signal();
 
+#if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION < 89 || (GTKMM_MINOR_VERSION == 89 && GTKMM_MICRO_VERSION < 4))) // GTKMM < 3.89.4
     main_loop.run();
+#else
+    Gtk::Main::run();
+#endif
 }
 
 #if defined(__APPLE__)
@@ -410,7 +483,7 @@ void GigEditState::run(gig::Instrument* pInstrument) {
     static bool main_loop_started = false;
     instrument = pInstrument;
     if (!main_loop_started) {
-#if defined(__APPLE__)
+#if defined(__APPLE__) && HAVE_LINUXSAMPLER
         // spawn GUI on main thread :
         //     On OS X the Gtk GUI can only be launched on a process's "main"
         //     thread. When trying to launch the Gtk GUI on any other thread,
