@@ -1,5 +1,5 @@
 /*                                                         -*- c++ -*-
- * Copyright (C) 2006 - 2015 Andreas Persson
+ * Copyright (C) 2006 - 2017 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -20,9 +20,17 @@
 #ifndef GIGEDIT_MAINWINDOW_H
 #define GIGEDIT_MAINWINDOW_H
 
-#include <gig.h>
+#ifdef LIBGIG_HEADER_FILE
+# include LIBGIG_HEADER_FILE(gig.h)
+# include LIBGIG_HEADER_FILE(Serialization.h)
+#else
+# include <gig.h>
+# include <Serialization.h>
+#endif
 
-#include <gtkmm/actiongroup.h>
+#include "compat.h"
+
+#include <gtkmm/box.h>
 #include <gtkmm/buttonbox.h>
 #include <gtkmm/dialog.h>
 #include <gtkmm/liststore.h>
@@ -32,10 +40,18 @@
 #include <gtkmm/radiomenuitem.h>
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/treestore.h>
-#include <gtkmm/uimanager.h>
+#include <gtkmm/treemodelfilter.h>
 #include <gtkmm/window.h>
 #include <gtkmm/statusbar.h>
 #include <gtkmm/image.h>
+#include <gtkmm/entry.h>
+#include <gtkmm/notebook.h>
+
+#if USE_GTKMM_BUILDER
+# include <gtkmm/builder.h>
+#else
+# include <gtkmm/uimanager.h> // deprecated in gtkmm >= 3.21.4
+#endif
 
 #include <sstream>
 
@@ -46,15 +62,23 @@
 #ifndef OLD_THREADS
 #include <glibmm/threads.h>
 #endif
+#include "ManagedWindow.h"
 
 class MainWindow;
 
-class PropDialog : public Gtk::Window,
+class PropDialog : public ManagedWindow,
                    public PropEditor<DLS::Info> {
 public:
     PropDialog();
     void set_info(DLS::Info* info);
     void set_file(gig::File* file);
+
+    // implementation for abstract methods of interface class "ManagedWindow"
+    virtual Settings::Property<int>* windowSettingX() { return &Settings::singleton()->filePropsWindowX; }
+    virtual Settings::Property<int>* windowSettingY() { return &Settings::singleton()->filePropsWindowY; }
+    virtual Settings::Property<int>* windowSettingWidth() { return &Settings::singleton()->filePropsWindowW; }
+    virtual Settings::Property<int>* windowSettingHeight() { return &Settings::singleton()->filePropsWindowH; }
+
 protected:
     ChoiceEntry<int> eFileFormat;
     StringEntry eName;
@@ -73,8 +97,8 @@ protected:
     StringEntry eSourceForm;
     StringEntry eCommissioned;
     StringEntry eSubject;
-    Gtk::VBox vbox;
-    Gtk::HButtonBox buttonBox;
+    VBox vbox;
+    HButtonBox buttonBox;
     Gtk::Button quitButton;
     Table table;
 
@@ -83,7 +107,7 @@ protected:
     void onFileFormatChanged();
 };
 
-class InstrumentProps : public Gtk::Window,
+class InstrumentProps : public ManagedWindow,
                         public PropEditor<gig::Instrument> {
 public:
     InstrumentProps();
@@ -93,6 +117,13 @@ public:
     sigc::signal<void>& signal_name_changed() {
         return sig_name_changed;
     }
+
+    // implementation for abstract methods of interface class "ManagedWindow"
+    virtual Settings::Property<int>* windowSettingX() { return &Settings::singleton()->instrPropsWindowX; }
+    virtual Settings::Property<int>* windowSettingY() { return &Settings::singleton()->instrPropsWindowY; }
+    virtual Settings::Property<int>* windowSettingWidth() { return &Settings::singleton()->instrPropsWindowW; }
+    virtual Settings::Property<int>* windowSettingHeight() { return &Settings::singleton()->instrPropsWindowH; }
+
 protected:
     void set_Name(const gig::String& name);
     void set_IsDrum(bool value);
@@ -100,8 +131,8 @@ protected:
     void set_MIDIProgram(uint32_t value);
 
     sigc::signal<void> sig_name_changed;
-    Gtk::VBox vbox;
-    Gtk::HButtonBox buttonBox;
+    VBox vbox;
+    HButtonBox buttonBox;
     Gtk::Button quitButton;
     Table table;
     StringEntry eName;
@@ -172,7 +203,7 @@ private:
     float progress;
 };
 
-class MainWindow : public Gtk::Window {
+class MainWindow : public ManagedWindow {
 public:
     MainWindow();
     virtual ~MainWindow();
@@ -198,9 +229,53 @@ public:
 
     sigc::signal<void, gig::Instrument*>& signal_switch_sampler_instrument();
 
+    sigc::signal<void, gig::Script*> signal_script_to_be_changed;
+    sigc::signal<void, gig::Script*> signal_script_changed;
+
+    // implementation for abstract methods of interface class "ManagedWindow"
+    virtual Settings::Property<int>* windowSettingX() { return &Settings::singleton()->mainWindowX; }
+    virtual Settings::Property<int>* windowSettingY() { return &Settings::singleton()->mainWindowY; }
+    virtual Settings::Property<int>* windowSettingWidth() { return &Settings::singleton()->mainWindowW; }
+    virtual Settings::Property<int>* windowSettingHeight() { return &Settings::singleton()->mainWindowH; }
+
 protected:
+#if USE_GTKMM_BUILDER
+    Glib::RefPtr<Gio::SimpleActionGroup> m_actionGroup;
+    Glib::RefPtr<Gtk::Builder> m_uiManager;
+#else
     Glib::RefPtr<Gtk::ActionGroup> actionGroup;
     Glib::RefPtr<Gtk::UIManager> uiManager;
+#endif
+
+#if USE_GLIB_ACTION
+    Glib::RefPtr<Gio::SimpleAction> m_actionMIDIRules;
+
+    Glib::RefPtr<Gio::SimpleAction> m_actionCopyDimRgn;
+    Glib::RefPtr<Gio::SimpleAction> m_actionPasteDimRgn;
+    Glib::RefPtr<Gio::SimpleAction> m_actionAdjustClipboard;
+
+    Glib::RefPtr<Gio::SimpleAction> m_actionSampleProperties;
+    Glib::RefPtr<Gio::SimpleAction> m_actionAddSample;
+    Glib::RefPtr<Gio::SimpleAction> m_actionRemoveSample;
+    Glib::RefPtr<Gio::SimpleAction> m_actionViewSampleRefs;
+    Glib::RefPtr<Gio::SimpleAction> m_actionReplaceSample;
+    Glib::RefPtr<Gio::SimpleAction> m_actionAddSampleGroup;
+
+    Glib::RefPtr<Gio::SimpleAction> m_actionAddScriptGroup;
+    Glib::RefPtr<Gio::SimpleAction> m_actionAddScript;
+    Glib::RefPtr<Gio::SimpleAction> m_actionEditScript;
+    Glib::RefPtr<Gio::SimpleAction> m_actionRemoveScript;
+
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleCopySampleUnity;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleCopySampleTune;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleCopySampleLoop;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleStatusBar;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleRestoreWinDim;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleSaveWithTempFile;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleWarnOnExtensions;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleSyncSamplerSelection;
+    Glib::RefPtr<Gio::SimpleAction> m_actionToggleMoveRootNoteWithRegion;
+#endif
 
     Gtk::Statusbar m_StatusBar;
     Gtk::Label     m_AttachedStateLabel;
@@ -213,6 +288,19 @@ protected:
     InstrumentProps instrumentProps;
     MidiRules midiRules;
 
+    /**
+     * Ensures that the 2 signals MainWindow::dimreg_to_be_changed_signal and
+     * MainWindowv::dimreg_changed_signal are always triggered correctly as a
+     * pair. It behaves similar to a "mutex lock guard" design pattern.
+     */
+    class DimRegionChangeGuard : public SignalGuard<gig::DimensionRegion*> {
+    public:
+        DimRegionChangeGuard(MainWindow* w, gig::DimensionRegion* pDimReg) :
+        SignalGuard<gig::DimensionRegion*>(w->dimreg_to_be_changed_signal, w->dimreg_changed_signal, pDimReg)
+        {
+        }
+    };
+
     sigc::signal<void, gig::File*> file_structure_to_be_changed_signal;
     sigc::signal<void, gig::File*> file_structure_changed_signal;
     sigc::signal<void, std::list<gig::Sample*> > samples_to_be_removed_signal;
@@ -229,7 +317,9 @@ protected:
 
     sigc::signal<void, gig::Instrument*> switch_sampler_instrument_signal;
 
+#if !USE_GTKMM_BUILDER
     void on_instrument_selection_change(Gtk::RadioMenuItem* item);
+#endif
     void on_sel_change();
     void region_changed();
     void dimreg_changed();
@@ -242,6 +332,13 @@ protected:
     void on_saver_progress();
     void on_saver_error();
     void on_saver_finished();
+    void updateMacroMenu();
+    void onMacroSelected(int iMacro);
+    void setupMacros();
+    void onMacrosSetupChanged(const std::vector<Serialization::Archive>& macros);
+    void applyMacro(Serialization::Archive& macro);
+    void onScriptSlotsModified(gig::Instrument* pInstrument);
+    void bringToFront();
 
     void dimreg_all_dimregs_toggled();
     gig::Instrument* get_instrument();
@@ -251,23 +348,33 @@ protected:
     class ModelColumns : public Gtk::TreeModel::ColumnRecord {
     public:
         ModelColumns() {
+          add(m_col_nr);
           add(m_col_name);
           add(m_col_instr);
+          add(m_col_scripts);
         }
 
+        Gtk::TreeModelColumn<int> m_col_nr;
         Gtk::TreeModelColumn<Glib::ustring> m_col_name;
         Gtk::TreeModelColumn<gig::Instrument*> m_col_instr;
+        Gtk::TreeModelColumn<Glib::ustring> m_col_scripts;
     } m_Columns;
 
-    Gtk::VBox m_VBox;
+    VBox m_VBox;
     Gtk::HPaned m_HPaned;
 
     Gtk::ScrolledWindow m_ScrolledWindow;
 
     Gtk::TreeView m_TreeView;
     Glib::RefPtr<Gtk::ListStore> m_refTreeModel;
+    Glib::RefPtr<Gtk::TreeModelFilter> m_refTreeModelFilter; //FIXME: I really would love to get rid of TreeModelFilter, because it causes behavior conflicts with get_model() all over the place (see the respective comments regarding get_model()), however I found no other way to filter a treeview effectively.
 
-    Gtk::Menu* instrument_menu;
+#if USE_GTKMM_BUILDER
+    Gtk::Menu* menuMacro;
+#else
+    Gtk::Menu* instrument_menu; // kept for GTKMM 2 version only, will be completely removed in future
+#endif
+    Gtk::Menu* assign_scripts_menu;
 
     std::map<gig::Sample*,int> sample_ref_count;
 
@@ -327,15 +434,30 @@ protected:
     Gtk::TreeView m_TreeViewScripts;
     Glib::RefPtr<ScriptsTreeStore> m_refScriptsTreeModel;
 
-    Gtk::VBox dimreg_vbox;
-    Gtk::HBox dimreg_hbox;
+    VBox dimreg_vbox;
+    HBox dimreg_hbox;
     Gtk::Label dimreg_label;
     Gtk::CheckButton dimreg_all_regions;
     Gtk::CheckButton dimreg_all_dimregs;
     Gtk::CheckButton dimreg_stereo;
+
+    HBox legend_hbox;
+    Gtk::Label labelLegend;
+    Gtk::Image imageNoSample;
+    Gtk::Label labelNoSample;
+    Gtk::Image imageMissingSample;
+    Gtk::Label labelMissingSample;
+    Gtk::Image imageLooped;
+    Gtk::Label labelLooped;
+    Gtk::Image imageSomeLoops;
+    Gtk::Label labelSomeLoops;
     DimRegionEdit dimreg_edit;
 
+    VBox m_left_vbox;
     Gtk::Notebook m_TreeViewNotebook;
+    HBox m_searchField;
+    Gtk::Label m_searchLabel;
+    Gtk::Entry m_searchText;
 
     struct SampleImportItem {
         gig::Sample*  gig_sample;  // pointer to the gig::Sample to
@@ -344,7 +466,7 @@ protected:
         Glib::ustring sample_path; // file name of the sample to be
                                    // imported
     };
-    std::list<SampleImportItem> m_SampleImportQueue;
+    std::map<gig::Sample*, SampleImportItem> m_SampleImportQueue;
 
 
     void on_action_file_new();
@@ -358,14 +480,22 @@ protected:
     void show_midi_rules();
     void show_script_slots();
     void on_action_view_status_bar();
+    void on_auto_restore_win_dim();
+    void on_save_with_temporary_file();
     void on_action_refresh_all();
     void on_action_warn_user_on_extensions();
     void on_action_sync_sampler_instrument_selection();
     void on_action_move_root_note_with_region_moved();
     void on_action_help_about();
 
+    void on_notebook_tab_switched(void* page, guint page_num);
+
     // sample right-click popup actions
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION > 91 || (GTKMM_MINOR_VERSION == 91 && GTKMM_MICRO_VERSION >= 2))) // GTKMM >= 3.91.2
+    bool on_sample_treeview_button_release(Gdk::EventButton& button);
+#else
     void on_sample_treeview_button_release(GdkEventButton* button);
+#endif
     void on_action_sample_properties();
     void on_action_add_group();
     void on_action_add_sample();
@@ -375,7 +505,11 @@ protected:
     void on_action_remove_unused_samples();
 
     // script right-click popup actions
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION > 91 || (GTKMM_MINOR_VERSION == 91 && GTKMM_MICRO_VERSION >= 2))) // GTKMM >= 3.91.2
+    bool on_script_treeview_button_release(Gdk::EventButton& button);
+#else
     void on_script_treeview_button_release(GdkEventButton* button);
+#endif
     void on_action_add_script_group();
     void on_action_add_script();
     void on_action_edit_script();
@@ -389,10 +523,44 @@ protected:
     void show_intruments_tab();
     void show_scripts_tab();
 
+    void select_prev_instrument();
+    void select_next_instrument();
+    void select_instrument_by_dir(int dir);
+
+    void select_prev_region();
+    void select_next_region();
+
+    void select_next_dim_rgn_zone();
+    void select_prev_dim_rgn_zone();
+    void select_add_next_dim_rgn_zone();
+    void select_add_prev_dim_rgn_zone();
+    void select_prev_dimension();
+    void select_next_dimension();
+
+    Serialization::Archive m_serializationArchive; ///< Clipboard content.
+    std::vector<Serialization::Archive> m_macros; ///< User configured list of macros.
+
+    void copy_selected_dimrgn();
+    void paste_copied_dimrgn();
+    void adjust_clipboard_content();
+    void updateClipboardCopyAvailable();
+    void updateClipboardPasteAvailable();
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION > 91 || (GTKMM_MINOR_VERSION == 91 && GTKMM_MICRO_VERSION >= 2))) // GTKMM >= 3.91.2
+    void on_clipboard_owner_change(Gdk::EventOwnerChange& event);
+#else
+    void on_clipboard_owner_change(GdkEventOwnerChange* event);
+#endif
+    void on_clipboard_get(Gtk::SelectionData& selection_data, guint info);
+    void on_clipboard_clear();
+    void on_clipboard_received(const Gtk::SelectionData& selection_data);
+    void on_clipboard_received_targets(const std::vector<Glib::ustring>& targets);
+
     void add_instrument(gig::Instrument* instrument);
+#if !USE_GTKMM_BUILDER
     Gtk::RadioMenuItem* add_instrument_to_menu(const Glib::ustring& name,
                                                int position = -1);
     void remove_instrument_from_menu(int index);
+#endif
 
     ProgressDialog* progress_dialog;
     Loader* loader;
@@ -414,7 +582,11 @@ protected:
     bool file_save_as();
     bool check_if_savable();
 
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && (GTKMM_MINOR_VERSION > 91 || (GTKMM_MINOR_VERSION == 91 && GTKMM_MICRO_VERSION >= 2))) // GTKMM >= 3.91.2
+    bool on_button_release(Gdk::EventButton& button);
+#else
     void on_button_release(GdkEventButton* button);
+#endif
     void on_instruments_treeview_drag_begin(const Glib::RefPtr<Gdk::DragContext>& context);
     void on_instruments_treeview_drag_data_get(const Glib::RefPtr<Gdk::DragContext>&,
                                                Gtk::SelectionData& selection_data, guint, guint);
@@ -441,6 +613,7 @@ protected:
     void instrument_name_changed(const Gtk::TreeModel::Path& path,
                                  const Gtk::TreeModel::iterator& iter);
     void instr_name_changed_by_instr_props(Gtk::TreeModel::iterator& it);
+    bool instrument_row_visible(const Gtk::TreeModel::const_iterator& iter);
     sigc::connection instrument_name_connection;
 
     void on_action_combine_instruments();
@@ -457,11 +630,17 @@ protected:
     void __import_queued_samples();
     void __clear();
     void __refreshEntireGUI();
+    void updateScriptListOfMenu();
+    void assignScript(gig::Script* pScript);
 
     bool close_confirmation_dialog();
     bool leaving_shared_mode_dialog();
 
     Gtk::Menu* popup_menu;
+#if USE_GTKMM_BUILDER
+    Gtk::Menu* sample_popup;
+    Gtk::Menu* script_popup;
+#endif
 
     bool on_delete_event(GdkEventAny* event);
 
