@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2015 Andreas Persson
+ * Copyright (C) 2006-2017 Andreas Persson
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,23 +17,56 @@
  * 02110-1301 USA.
  */
 
+#include "compat.h"
+#include "global.h"
 #include "regionchooser.h"
 
 #include <algorithm>
+#include <assert.h>
 
 #include <cairomm/context.h>
 #include <gdkmm/general.h>
+#if HAS_GDKMM_SEAT
+# include <gdkmm/seat.h>
+#endif
 #include <gdkmm/cursor.h>
-#include <gtkmm/stock.h>
+#if HAS_GTKMM_STOCK
+# include <gtkmm/stock.h>
+#endif
+#include <gdkmm/pixbuf.h>
 #include <gtkmm/spinbutton.h>
 #include <gtkmm/dialog.h>
 
-#include "global.h"
 #include "Settings.h"
+#include "gfx/builtinpix.h"
 
 #define REGION_BLOCK_HEIGHT             30
 #define KEYBOARD_HEIGHT                 40
 
+struct RegionFeatures {
+    int sampleRefs;
+    int loops;
+    int validDimRegs;
+
+    RegionFeatures() {
+        sampleRefs = loops = validDimRegs = 0;
+    }
+};
+
+static RegionFeatures regionFeatures(gig::Region* rgn) {
+    RegionFeatures f;
+    for (int i = 0; i < rgn->DimensionRegions; ++i) {
+        gig::DimensionRegion* dr = rgn->pDimensionRegions[i];
+        DimensionCase c = dimensionCaseOf(dr);
+        if (!isUsedCase(c, rgn)) continue;
+        f.validDimRegs++;
+        if (dr->pSample) f.sampleRefs++;
+        // the user doesn't care about loop if there is no valid sample reference
+        if (dr->pSample && dr->SampleLoops) f.loops++;
+    }
+    return f;
+}
+
 void SortedRegions::update(gig::Instrument* instrument) {
     // Usually, the regions in a gig file are ordered after their key
     // range, but there are files where they are not. The
@@ -55,7 +88,7 @@ gig::Region* SortedRegions::first() {
 }
 
 gig::Region* SortedRegions::next() {
-    region_iterator++;
+    ++region_iterator;
     return region_iterator == regions.end() ? 0 : *region_iterator;
 }
 
@@ -63,15 +96,56 @@ gig::Region* SortedRegions::next() {
 
 RegionChooser::RegionChooser() :
     activeKeyColor("red"),
-    red("#8070ff"),
+    blue("#4796ff"),
     grey1("grey69"),
     white("white"),
     black("black"),
     m_VirtKeybModeChoice(_("Virtual Keyboard Mode")),
-    currentActiveKey(-1)
+    currentActiveKey(-1),
+    modifyallregions(false)
 {
     set_size_request(500, KEYBOARD_HEIGHT + REGION_BLOCK_HEIGHT);
 
+    loadBuiltInPix();
+
+    // create blue hatched pattern
+    {
+        const int width = blueHatchedPattern->get_width();
+        const int height = blueHatchedPattern->get_height();
+        const int stride = blueHatchedPattern->get_rowstride();
+
+        // manually convert from RGBA to ARGB
+        this->blueHatchedPatternARGB = blueHatchedPattern->copy();
+        const int pixelSize = stride / width;
+        const int totalPixels = width * height;
+        assert(pixelSize == 4);
+        unsigned char* ptr = this->blueHatchedPatternARGB->get_pixels();
+        for (int iPixel = 0; iPixel < totalPixels; ++iPixel, ptr += pixelSize) {
+            const unsigned char r = ptr[0];
+            const unsigned char g = ptr[1];
+            const unsigned char b = ptr[2];
+            const unsigned char a = ptr[3];
+            ptr[0] = b;
+            ptr[1] = g;
+            ptr[2] = r;
+            ptr[3] = a;
+        }
+
+        Cairo::RefPtr<Cairo::ImageSurface> imageSurface = Cairo::ImageSurface::create(
+#if HAS_CAIROMM_CPP11_ENUMS
+            this->blueHatchedPatternARGB->get_pixels(), Cairo::Surface::Format::ARGB32, width, height, stride
+#else
+            this->blueHatchedPatternARGB->get_pixels(), Cairo::FORMAT_ARGB32, width, height, stride
+#endif
+        );
+        this->blueHatchedSurfacePattern = Cairo::SurfacePattern::create(imageSurface);
+#if HAS_CAIROMM_CPP11_ENUMS
+        this->blueHatchedSurfacePattern->set_extend(Cairo::Pattern::Extend::REPEAT);
+#else
+        this->blueHatchedSurfacePattern->set_extend(Cairo::EXTEND_REPEAT);
+#endif
+    }
+
     instrument = 0;
     region = 0;
     resize.active = false;
@@ -103,7 +177,22 @@ RegionChooser::RegionChooser() :
     m_VirtKeybPropsBox.show();
     for (int i = 0 ; i < 128 ; i++) key_pressed[i] = false;
 
-    actionGroup = Gtk::ActionGroup::create();
+    actionGroup = ActionGroup::create();
+#if USE_GLIB_ACTION
+    actionGroup->add_action(
+        "Properties", sigc::mem_fun(*this, &RegionChooser::show_region_properties)
+    );
+    actionGroup->add_action(
+        "Remove", sigc::mem_fun(*this, &RegionChooser::delete_region)
+    );
+    actionGroup->add_action(
+        "Add", sigc::mem_fun(*this, &RegionChooser::add_region)
+    );
+    actionGroup->add_action(
+        "Dimensions", sigc::mem_fun(*this, &RegionChooser::manage_dimensions)
+    );
+    insert_action_group("PopupMenuInsideRegion", actionGroup);
+#else
     actionGroup->add(Gtk::Action::create("Properties",
                                          Gtk::Stock::PROPERTIES),
                      sigc::mem_fun(*this,
@@ -114,7 +203,50 @@ RegionChooser::RegionChooser() :
                      sigc::mem_fun(*this, &RegionChooser::add_region));
     actionGroup->add(Gtk::Action::create("Dimensions", _("Dimensions...")),
                      sigc::mem_fun(*this, &RegionChooser::manage_dimensions));
+#endif
 
+#if USE_GTKMM_BUILDER
+    uiManager = Gtk::Builder::create();
+    Glib::ustring ui_info =
+        "<interface>"
+        "  <menu id='menu-PopupMenuInsideRegion'>"
+        "    <section>"
+        "      <item>"
+        "        <attribute name='label' translatable='yes'>Properties</attribute>"
+        "        <attribute name='action'>PopupMenuInsideRegion.Properties</attribute>"
+        "      </item>"
+        "      <item>"
+        "        <attribute name='label' translatable='yes'>Dimensions</attribute>"
+        "        <attribute name='action'>PopupMenuInsideRegion.Dimensions</attribute>"
+        "      </item>"
+        "      <item>"
+        "        <attribute name='label' translatable='yes'>Remove</attribute>"
+        "        <attribute name='action'>PopupMenuInsideRegion.Remove</attribute>"
+        "      </item>"
+        "    </section>"
+        "  </menu>"
+        "  <menu id='menu-PopupMenuOutsideRegion'>"
+        "    <section>"
+        "      <item>"
+        "        <attribute name='label' translatable='yes'>Add</attribute>"
+        "        <attribute name='action'>PopupMenuInsideRegion.Add</attribute>"
+        "      </item>"
+        "    </section>"
+        "  </menu>"
+        "</interface>";
+    uiManager->add_from_string(ui_info);
+    
+    popup_menu_inside_region = new Gtk::Menu(
+         Glib::RefPtr<Gio::Menu>::cast_dynamic(
+             uiManager->get_object("menu-PopupMenuInsideRegion")
+         )
+    );
+    popup_menu_outside_region = new Gtk::Menu(
+         Glib::RefPtr<Gio::Menu>::cast_dynamic(
+             uiManager->get_object("menu-PopupMenuOutsideRegion")
+         )
+    );
+#else
     uiManager = Gtk::UIManager::create();
     uiManager->insert_action_group(actionGroup);
     Glib::ustring ui_info =
@@ -135,8 +267,15 @@ RegionChooser::RegionChooser() :
     popup_menu_outside_region = dynamic_cast<Gtk::Menu*>(
         uiManager->get_widget("/PopupMenuOutsideRegion"));
 
+#endif // USE_GTKMM_BUILDER
+
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+# warning GTKMM4 event registration code missing for regionchooser!
+    //add_events(Gdk::EventMask::BUTTON_PRESS_MASK);
+#else
     add_events(Gdk::BUTTON_PRESS_MASK | Gdk::BUTTON_RELEASE_MASK |
                Gdk::POINTER_MOTION_MASK | Gdk::POINTER_MOTION_HINT_MASK);
+#endif
 
     dimensionManager.region_to_be_changed_signal.connect(
         region_to_be_changed_signal.make_slot()
@@ -162,6 +301,12 @@ RegionChooser::~RegionChooser()
 {
 }
 
+void RegionChooser::setModifyAllRegions(bool b) {
+    modifyallregions = b;
+    // redraw required parts
+    queue_draw();
+}
+
 void RegionChooser::invalidate_key(int key) {
     const int h = KEYBOARD_HEIGHT;
     const int w = get_width() - 1;
@@ -209,7 +354,13 @@ bool RegionChooser::on_draw(const Cairo::RefPtr<Cairo::Context>& cr) {
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
     const Gdk::Color bg = get_style()->get_bg(Gtk::STATE_NORMAL);
 #else
+#if GTKMM_MAJOR_VERSION > 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION > 22)
+    GdkRGBA gdkBgRGBA;
+    gtk_style_context_get_background_color(get_style_context()->gobj(), &gdkBgRGBA);
+    const Gdk::RGBA bg = Glib::wrap(&gdkBgRGBA, true);
+# else
     const Gdk::RGBA bg = get_style_context()->get_background_color();
+# endif
 #endif
     Gdk::Cairo::set_source_rgba(cr, bg);
     cr->paint();
@@ -312,7 +463,13 @@ void RegionChooser::draw_regions(const Cairo::RefPtr<Cairo::Context>& cr,
                 cr->line_to(x3, h1 - 0.5);
                 cr->stroke();
 
-                Gdk::Cairo::set_source_rgba(cr, region == r ? red : white);
+                if (region == r)
+                    Gdk::Cairo::set_source_rgba(cr, blue);
+                else if (modifyallregions)
+                    cr->set_source(blueHatchedSurfacePattern);
+                else
+                    Gdk::Cairo::set_source_rgba(cr, white);
+
                 cr->rectangle(x3 + 1, 1, x2 - x3 - 1, h1 - 2);
                 cr->fill();
                 Gdk::Cairo::set_source_rgba(cr, black);
@@ -321,6 +478,47 @@ void RegionChooser::draw_regions(const Cairo::RefPtr<Cairo::Context>& cr,
         }
     }
 
+    for (gig::Region* r = regions.first() ; r ; r = regions.next()) {
+        int x = key_to_x(r->KeyRange.low, w);
+        int x2 = key_to_x(r->KeyRange.high + 1, w);
+
+        RegionFeatures features = regionFeatures(r);
+
+        const bool bShowLoopSymbol = features.loops > 0;
+        const bool bShowSampleRefSymbol = features.sampleRefs < features.validDimRegs;
+        if (bShowLoopSymbol || bShowSampleRefSymbol) {
+            const int margin = 2;
+            const int wRgn = x2 - x;
+            //printf("x=%d x2=%d wRgn=%d\n", x, x2, wRgn);
+
+            cr->save();
+            cr->set_line_width(1);
+            cr->rectangle(x, 1, wRgn, h1 - 1);
+            cr->clip();
+            if (bShowSampleRefSymbol) {
+                const int wPic = 8;
+                const int hPic = 8;
+                Gdk::Cairo::set_source_pixbuf(
+                    cr, (features.sampleRefs) ? yellowDot : redDot,
+                    x + (wRgn-wPic)/2.f,
+                    (bShowLoopSymbol) ? margin : (h1-hPic)/2.f
+                );
+                cr->paint();
+            }
+            if (bShowLoopSymbol) {
+                const int wPic = 12;
+                const int hPic = 14;
+                Gdk::Cairo::set_source_pixbuf(
+                    cr, (features.loops == features.validDimRegs) ? blackLoop : grayLoop,
+                    x + (wRgn-wPic)/2.f,
+                    (bShowSampleRefSymbol) ? h1 - hPic - margin : (h1-hPic)/2.f
+                );
+                cr->paint();
+            }
+            cr->restore();
+        }
+    }
+
     for (gig::Region* r = regions.first() ; r ; r = regions.next()) {
         int x = key_to_x(r->KeyRange.low, w);
 
@@ -342,9 +540,8 @@ void RegionChooser::draw_regions(const Cairo::RefPtr<Cairo::Context>& cr,
         layout->set_width(get_width() * Pango::SCALE);
         //layout->set_height(get_height() * Pango::SCALE);
         layout->set_spacing(10);
-        Gdk::Cairo::set_source_rgba(cr, red);        
+        Gdk::Cairo::set_source_rgba(cr, blue);
         // get the text dimensions
-        Pango::Rectangle rect = layout->get_logical_extents();
         int text_width, text_height;
         layout->get_pixel_size(text_width, text_height);
         cr->move_to(0, (REGION_BLOCK_HEIGHT - text_height) / 2);
@@ -462,7 +659,11 @@ bool RegionChooser::on_button_release_event(GdkEventButton* event)
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
         get_window()->pointer_ungrab(event->time);
 #else
+# if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
         Glib::wrap(event->device, true)->ungrab(event->time);
+# else
+        gdk_device_ungrab(Glib::wrap(event->device, true)->gobj(), event->time);
+# endif
 #endif
         resize.active = false;
 
@@ -474,7 +675,11 @@ bool RegionChooser::on_button_release_event(GdkEventButton* event)
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
         get_window()->pointer_ungrab(event->time);
 #else
+# if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
         Glib::wrap(event->device, true)->ungrab(event->time);
+# else
+        gdk_device_ungrab(Glib::wrap(event->device, true)->gobj(), event->time);
+# endif
 #endif
         move.active = false;
 
@@ -482,7 +687,16 @@ bool RegionChooser::on_button_release_event(GdkEventButton* event)
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
             get_window()->set_cursor(Gdk::Cursor(Gdk::SB_H_DOUBLE_ARROW));
 #else
-            get_window()->set_cursor(Gdk::Cursor::create(Gdk::SB_H_DOUBLE_ARROW));
+            get_window()->set_cursor(
+# if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
+                Gdk::Cursor::create(Gdk::SB_H_DOUBLE_ARROW)
+# else
+                Gdk::Cursor::create(
+                    Glib::wrap(event->device, true)->get_seat()->get_display(),
+                    Gdk::SB_H_DOUBLE_ARROW
+                )
+# endif
+            );
 #endif
             cursor_is_resize = true;
         }
@@ -578,6 +792,7 @@ bool RegionChooser::on_button_press_event(GdkEventButton* event)
                                        Gdk::Cursor(Gdk::SB_H_DOUBLE_ARROW),
                                        event->time);
 #else
+# if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
             Glib::wrap(event->device, true)->grab(get_window(),
                                                   Gdk::OWNERSHIP_NONE,
                                                   false,
@@ -586,6 +801,21 @@ bool RegionChooser::on_button_press_event(GdkEventButton* event)
                                                   Gdk::POINTER_MOTION_HINT_MASK,
                                                   Gdk::Cursor::create(Gdk::SB_H_DOUBLE_ARROW),
                                                   event->time);
+# else
+            gdk_device_grab(
+                Glib::wrap(event->device, true)->gobj(),
+                get_window()->gobj(),
+                GDK_OWNERSHIP_NONE,
+                false,
+                GdkEventMask(GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK |
+                             GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK),
+                Gdk::Cursor::create(
+                    Glib::wrap(event->device, true)->get_seat()->get_display(),
+                    Gdk::SB_H_DOUBLE_ARROW
+                )->gobj(),
+                event->time
+            );
+# endif
 #endif
             resize.active = true;
         } else {
@@ -604,14 +834,30 @@ bool RegionChooser::on_button_press_event(GdkEventButton* event)
                                            Gdk::Cursor(Gdk::FLEUR),
                                            event->time);
 #else
-                Glib::wrap(event->device, true)->grab(get_window(),
-                                                      Gdk::OWNERSHIP_NONE,
-                                                      false,
-                                                      Gdk::BUTTON_RELEASE_MASK |
-                                                      Gdk::POINTER_MOTION_MASK |
-                                                      Gdk::POINTER_MOTION_HINT_MASK,
-                                                      Gdk::Cursor::create(Gdk::FLEUR),
-                                                      event->time);
+# if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
+            Glib::wrap(event->device, true)->grab(get_window(),
+                                                  Gdk::OWNERSHIP_NONE,
+                                                  false,
+                                                  Gdk::BUTTON_RELEASE_MASK |
+                                                  Gdk::POINTER_MOTION_MASK |
+                                                  Gdk::POINTER_MOTION_HINT_MASK,
+                                                  Gdk::Cursor::create(Gdk::FLEUR),
+                                                  event->time);
+# else
+            gdk_device_grab(
+                Glib::wrap(event->device, true)->gobj(),
+                get_window()->gobj(),
+                GDK_OWNERSHIP_NONE,
+                false,
+                GdkEventMask(GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK |
+                             GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK),
+                Gdk::Cursor::create(
+                    Glib::wrap(event->device, true)->get_seat()->get_display(),
+                    Gdk::FLEUR
+                )->gobj(),
+                event->time
+            );
+# endif
 #endif
                 move.active = true;
                 move.offset = event->x - key_to_x(region->KeyRange.low, w);
@@ -637,6 +883,60 @@ void RegionChooser::set_region(gig::Region* region) {
     dimensionManager.set_region(region);
 }
 
+void RegionChooser::select_next_region() {
+    if (!instrument) return;
+    if (!region) {
+        for (int i = 0; i < 128; ++i) {
+            ::gig::Region* rgn = instrument->GetRegion(i);
+            if (rgn) {
+                set_region(rgn);
+                return;
+            }
+        }
+    } else {
+        bool currentFound = false;
+        for (int i = 0; i < 128; ++i) {
+            ::gig::Region* rgn = instrument->GetRegion(i);
+            if (!rgn) continue;
+            if (currentFound) {
+                if (rgn != region) {
+                    set_region(rgn);
+                    return;
+                }
+            } else {
+                if (rgn == region) currentFound = true;
+            }
+        }
+    }
+}
+
+void RegionChooser::select_prev_region() {
+    if (!instrument) return;
+    if (!region) {
+        for (int i = 0; i < 128; ++i) {
+            ::gig::Region* rgn = instrument->GetRegion(i);
+            if (rgn) {
+                set_region(rgn);
+                return;
+            }
+        }
+    } else {
+        bool currentFound = false;
+        for (int i = 127; i >= 0; --i) {
+            ::gig::Region* rgn = instrument->GetRegion(i);
+            if (!rgn) continue;
+            if (currentFound) {
+                if (rgn != region) {
+                    set_region(rgn);
+                    return;
+                }
+            } else {
+                if (rgn == region) currentFound = true;
+            }
+        }
+    }
+}
+
 void RegionChooser::motion_resize_region(int x, int y)
 {
     const int w = get_width() - 1;
@@ -685,7 +985,8 @@ void RegionChooser::motion_resize_region(int x, int y)
 
         update_after_resize();
 
-        get_window()->invalidate_rect(rect, false);
+        //get_window()->invalidate_rect(rect, false);
+        get_window()->invalidate(false); // repaint entire region, otherwise it would create visual artifacts
     }
 }
 
@@ -774,8 +1075,14 @@ bool RegionChooser::on_motion_notify_event(GdkEventMotion* event)
 {
     Glib::RefPtr<Gdk::Window> window = get_window();
     int x, y;
+#if HAS_GDKMM_SEAT
+    x = event->x;
+    y = event->y;
+    Gdk::ModifierType state = Gdk::ModifierType(event->state);
+#else
     Gdk::ModifierType state = Gdk::ModifierType(0);
     window->get_pointer(x, y, state);
+#endif
 
     // handle virtual MIDI keyboard
     if (m_VirtKeybModeChoice.get_value() != VIRT_KEYBOARD_MODE_CHORD &&
@@ -806,7 +1113,16 @@ bool RegionChooser::on_motion_notify_event(GdkEventMotion* event)
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
                 window->set_cursor(Gdk::Cursor(Gdk::SB_H_DOUBLE_ARROW));
 #else
-                window->set_cursor(Gdk::Cursor::create(Gdk::SB_H_DOUBLE_ARROW));
+                window->set_cursor(
+# if GTKMM_MAJOR_VERSION < 3 || (GTKMM_MAJOR_VERSION == 3 && GTKMM_MINOR_VERSION < 20)
+                    Gdk::Cursor::create(Gdk::SB_H_DOUBLE_ARROW)
+# else
+                    Gdk::Cursor::create(
+                        Glib::wrap(event->device, true)->get_seat()->get_display(),
+                        Gdk::SB_H_DOUBLE_ARROW
+                    )
+# endif
+                );
 #endif
                 cursor_is_resize = true;
             }
@@ -886,7 +1202,11 @@ void RegionChooser::show_region_properties()
     // add "Keygroup" checkbox
     Gtk::CheckButton checkBoxKeygroup(_("Member of a Keygroup (Exclusive Group)"));
     checkBoxKeygroup.set_active(region->KeyGroup);
+#if USE_GTKMM_BOX
+    dialog.get_content_area()->pack_start(checkBoxKeygroup);
+#else
     dialog.get_vbox()->pack_start(checkBoxKeygroup);
+#endif
     checkBoxKeygroup.show();
     // add "Keygroup" spinbox
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
@@ -896,12 +1216,24 @@ void RegionChooser::show_region_properties()
     Gtk::SpinButton spinBox(Gtk::Adjustment::create(1, 1, 999));
 #endif
     if (region->KeyGroup) spinBox.set_value(region->KeyGroup);
+#if USE_GTKMM_BOX
+    dialog.get_content_area()->pack_start(spinBox);
+#else
     dialog.get_vbox()->pack_start(spinBox);
+#endif
     spinBox.show();
     // add OK and CANCEL buttons to the dialog
+#if HAS_GTKMM_STOCK
     dialog.add_button(Gtk::Stock::OK, 0);
     dialog.add_button(Gtk::Stock::CANCEL, 1);
+#else
+    dialog.add_button(_("_OK"), 0);
+    dialog.add_button(_("_Cancel"), 1);
+#endif
+    dialog.set_position(Gtk::WIN_POS_MOUSE);
+#if HAS_GTKMM_SHOW_ALL_CHILDREN
     dialog.show_all_children();
+#endif
     if (!dialog.run()) { // OK selected ...
         region->KeyGroup =
             (checkBoxKeygroup.get_active()) ? spinBox.get_value_as_int() : 0;
